require 'oauth/request_proxy/action_controller_request'
require 'colorize'

module LtiProvider
  class LtiController < LtiProvider::LtiApplicationController
    skip_before_action :require_lti_launch
    skip_before_action :verify_authenticity_token, only: :launch, if: :digital_library?

    def launch
      tool_key = params['oauth_consumer_key']
      if canvas_url_domain && ToolLtiKey.table_exists?
        tool_lti_key = ToolLtiKey.where(key: tool_key).where('lower(canvas_url) LIKE :q', q: "%#{canvas_url_domain.downcase}%" ).first
        tool_secret = tool_lti_key&.secret or LtiProvider::Config.secret
      else
        tool_secret = LtiProvider::Config.secret
      end
      
      provider = IMS::LTI::ToolProvider.new(tool_key, tool_secret, params)
      launch = Launch.initialize_from_request(provider, request)

      if !launch.valid_provider?
        msg = "#{launch.lti_errormsg} Please be sure you are launching this tool from the link provided in Qalam."
        return show_error msg
      elsif launch.save
        session[:cookie_test] = true
        ### QALAM ###
        if params[:timetable_date] && params[:section_room]
          redirect_to cookie_test_path(nonce: launch.nonce, timetable_date: params[:timetable_date], section_room: params[:section_room])
        else
          redirect_to cookie_test_path(nonce: launch.nonce)
        end
        ### END ###
      else
        # if params[:qalam_url] && params[:launch_tool] == 'digital_library'
        #   redirect_to main_app.root_path and return 
        # else
          return show_error "Unable to launch #{LtiProvider::XmlConfig.tool_title}. Please check your External Tools configuration and try again."
        # end
      end
    end

    def cookie_test
      @launch = Launch.where("created_at > ?", 5.minutes.ago).find_by_nonce(params[:nonce])

      if session[:cookie_test]
        # success!!! we've got a session!
        consume_launch
      else
        # if @launch.launch_tool && @launch.launch_tool == 'digital_library'
        #   redirect_to main_app.root_path and return 
        # else
          render and return
        # end
      end
    end

    def consume_launch
      if @launch
        [:account_id, :course_name, :course_id, :canvas_url, :tool_consumer_instance_guid,
         :user_id, :user_name, :user_roles, :user_avatar_url, :launch_tool].each do |attribute|
          session[attribute] = @launch.public_send(attribute)
        end

        ### QALAM ###
        if params[:timetable_date] && params[:section_room]
          session[:timetable_date] = params[:timetable_date]
          session[:section_room] = params[:section_room]
        end
        ### END ###
        
        @launch.destroy

        # if @launch.launch_tool && @launch.launch_tool == 'digital_library'
        #   redirect_to main_app.root_path and return 
        # else
          redirect_to main_app.root_path and return
        # end
      else
        # if @launch.launch_tool && @launch.launch_tool == 'digital_library'
        #   redirect_to main_app.root_path and return 
        # else
          return show_error "The tool was not launched successfully. Please try again."
        # end
      end
    end

    def configure
      respond_to do |format|
        format.xml do
          render xml: Launch.xml_config(lti_launch_url)
        end
      end
    end

    protected
      def show_error(message)
        render plain: message
      end

    private
      def canvas_url_domain
        if params['launch_presentation_return_url']
          uri = URI.parse(params['launch_presentation_return_url'])
          domain = "#{uri.scheme}://#{uri.host}"
          domain += ":#{uri.port}" unless uri.port.nil? || [80, 443].include?(uri.port.to_i)
          return domain
        end
      end

      def digital_library? 
        session[:qalam_url] ? true : false
      end
  end
end