module LtiProvider
  module LtiApplication
    extend ActiveSupport::Concern

    module ClassMethods
    end

    included do
      before_action :require_lti_launch
    end

    protected
    def require_lti_launch
      if canvas_url.blank? || user_id.blank?
        if session[:qalam_url]
          reset_session
          prompt_for_launch_authorized
        else
          reset_session
          prompt_for_launch
        end
      end
    end

    def prompt_for_launch
      render plain: 'Please launch this tool from Qalam and then try again.'
    end

    def prompt_for_launch_authorized
      render plain: 'You need to be login on Qalam.'
    end

    def canvas_url
      session[:canvas_url]
    end

    def user_id
      session[:user_id]
    end

    def current_course_id
      session[:course_id]
    end

    def tool_consumer_instance_guid
      session[:tool_consumer_instance_guid]
    end

    def course_launch?
      current_course_id.present?
    end

    def current_account_id
      session[:account_id]
    end

    def account_launch?
      current_account_id.present?
    end

    def user_roles
      session[:user_roles]
    end

    ### QALAM ###
    def launch_tool
      session[:launch_tool]
    end

    def student_launch?
      user_roles&.split(',')&.include? 'urn:lti:role:ims/lis/Learner'
    end

    def teacher_launch?
      user_roles&.split(',')&.include? 'urn:lti:role:ims/lis/Instructor'
    end

    def account_admin_launch?
      user_roles&.split(',')&.include? 'urn:lti:instrole:ims/lis/Administrator'
    end

    def root_admin_launch?
      user_roles&.split(',')&.include? 'urn:lti:sysrole:ims/lis/SysAdmin'
    end
    ### END ###

    def not_acceptable
      render plain: "Unable to process request", status: 406
    end
  end
end